/// <reference types="node" />
export declare enum TlvType {
    SignedInt = 0,
    UnsignedInt = 4,
    Boolean = 8,
    Float = 10,
    Double = 11,
    String = 12,
    ByteString = 16,
    Null = 20,
    Structure = 21,
    Array = 22,
    List = 23,
    EndOfContainer = 24
}
export declare const enum ElementSize {
    Byte1 = 0,
    Byte2 = 1,
    Byte4 = 2,
    Byte8 = 3
}
export interface Element {
    type: TlvType;
    value: any;
    tag: TlvTag;
}
export declare class TlvTag {
    readonly profile: number;
    readonly id: number;
    constructor(profile: number, id: number);
    toNumber(): number;
    toString(): string;
    equals({ profile, id }: TlvTag): boolean;
    isAnonymous(): boolean;
    isContextual(): boolean;
    isCommon(): boolean;
    static readonly Anonymous: TlvTag;
    static readonly contextual: (id: number) => TlvTag;
}
export declare class TlvCodec {
    static decodeElement(data: Buffer): Element;
    private static decodeElementInternal;
    private static decodeTagAndTypeSize;
    private static decodeTag;
    private static decodeValue;
    private static decodeContainer;
    static encodeElement(element: Element | undefined): Buffer;
    private static encodeElementInternal;
    private static encodeString;
    private static encodeByteString;
    private static encodeUnsignedInt;
    private static encodeSignedInt;
    private static encodeBoolean;
    private static encodeContainer;
    private static encodeControlByteAndTag;
    private static encodeUnsignedIntBytes;
    private static encodeSignedIntBytes;
    private static getUnsignedIntSize;
    private static getSignedIntSize;
}
