/// <reference types="node" />
import { Merge } from "../util/Type";
import { TlvType, Element, TlvTag } from "./TlvCodec";
export interface Template<T> {
    tlvType?: TlvType;
    readAsBigint?: boolean;
    jsType?: T;
}
export interface ArrayTemplate<T> extends Template<T> {
    itemTemplate: Template<T>;
}
export interface ObjectTemplate<T> extends Template<T> {
    fieldTemplates: FieldTemplates;
}
export interface TaggedTemplate<T> extends Template<T> {
    tag?: TlvTag;
}
export interface Field<T> extends TaggedTemplate<T> {
    optional: false;
}
export interface OptionalField<T> extends TaggedTemplate<T> {
    optional: true;
}
declare type FieldTemplates = {
    [key: string]: Field<any> | OptionalField<any>;
};
declare type OptionalKeys<T extends object> = {
    [K in keyof T]: T[K] extends OptionalField<any> ? K : never;
}[keyof T];
declare type RequiredKeys<T extends object> = {
    [K in keyof T]: T[K] extends OptionalField<any> ? never : K;
}[keyof T];
export declare type JsType<Type> = Type extends Template<infer T> ? T : never;
export declare type TypeFromFieldTemplates<T extends FieldTemplates> = Merge<{
    [P in RequiredKeys<T>]: JsType<T[P]>;
}, {
    [P in OptionalKeys<T>]?: JsType<T[P]>;
}>;
export declare const StringT: Template<string>;
export declare const BooleanT: Template<boolean>;
export declare const ByteStringT: Template<Buffer>;
export declare const UnsignedIntT: Template<number>;
export declare const UnsignedLongT: Template<bigint>;
export declare const AnyT: Template<any>;
export declare const ArrayT: <T>(itemTemplate: Template<T>) => Template<T[]>;
export declare const ObjectT: <F extends FieldTemplates>(fieldTemplates: F, tlvType?: TlvType) => Template<Merge<{ [P in RequiredKeys<F>]: JsType<F[P]>; }, { [P_1 in OptionalKeys<F>]?: JsType<F[P_1]> | undefined; }, { [P in RequiredKeys<F>]: JsType<F[P]>; } & { [P_1 in OptionalKeys<F>]?: JsType<F[P_1]> | undefined; }>>;
export declare const Field: <T>(id: number, type: Template<T>) => Field<T>;
export declare const OptionalField: <T>(id: number, type: Template<T>) => OptionalField<T>;
export declare class TlvObjectCodec {
    static decode<T>(bytes: Buffer, template: TaggedTemplate<T>): T;
    static decodeElement<T>(element: Element | undefined, template: TaggedTemplate<T>): T;
    private static decodeInternal;
    private static decodeArray;
    private static decodeObject;
    static encode<T>(value: T, template: TaggedTemplate<T>): Buffer;
    static encodeElement<T>(value: T, template: TaggedTemplate<T>): Element | undefined;
    private static encodeInternal;
    private static encodeArray;
    private static encodeObject;
}
export {};
