"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TlvObjectCodec = exports.OptionalField = exports.Field = exports.ObjectT = exports.ArrayT = exports.AnyT = exports.UnsignedLongT = exports.UnsignedIntT = exports.ByteStringT = exports.BooleanT = exports.StringT = void 0;
const TlvCodec_1 = require("./TlvCodec");
;
;
;
;
;
;
exports.StringT = { tlvType: TlvCodec_1.TlvType.String };
exports.BooleanT = { tlvType: TlvCodec_1.TlvType.Boolean };
exports.ByteStringT = { tlvType: TlvCodec_1.TlvType.ByteString };
exports.UnsignedIntT = { tlvType: TlvCodec_1.TlvType.UnsignedInt };
exports.UnsignedLongT = { tlvType: TlvCodec_1.TlvType.UnsignedInt, readAsBigint: true };
exports.AnyT = {};
const ArrayT = (itemTemplate) => ({ tlvType: TlvCodec_1.TlvType.Array, itemTemplate });
exports.ArrayT = ArrayT;
const ObjectT = (fieldTemplates, tlvType = TlvCodec_1.TlvType.Structure) => ({ tlvType, fieldTemplates });
exports.ObjectT = ObjectT;
const Field = (id, type) => (Object.assign(Object.assign({}, type), { tag: TlvCodec_1.TlvTag.contextual(id), optional: false }));
exports.Field = Field;
const OptionalField = (id, type) => (Object.assign(Object.assign({}, (0, exports.Field)(id, type)), { optional: true }));
exports.OptionalField = OptionalField;
class TlvObjectCodec {
    static decode(bytes, template) {
        return this.decodeElement(TlvCodec_1.TlvCodec.decodeElement(bytes), template);
    }
    static decodeElement(element, template) {
        return this.decodeInternal(element, template);
    }
    static decodeInternal(element, template) {
        const { tlvType: expectedType, tag: expectedTag = TlvCodec_1.TlvTag.Anonymous, readAsBigint } = template;
        if (element === undefined)
            return undefined;
        const { tag, value, type } = element;
        if (!tag.equals(expectedTag))
            throw new Error(`Unexpected tag ${tag}. Expected was ${expectedTag}`);
        if (expectedType === undefined) {
            return { tag: TlvCodec_1.TlvTag.Anonymous, value, type };
        }
        if (type !== expectedType)
            throw new Error(`Unexpected type ${type}. Expected was ${expectedType}`);
        switch (type) {
            case TlvCodec_1.TlvType.Boolean:
            case TlvCodec_1.TlvType.String:
            case TlvCodec_1.TlvType.ByteString:
            case TlvCodec_1.TlvType.Float:
            case TlvCodec_1.TlvType.Double:
            case TlvCodec_1.TlvType.Null:
                return value;
            case TlvCodec_1.TlvType.UnsignedInt:
            case TlvCodec_1.TlvType.SignedInt:
                if (readAsBigint)
                    return BigInt(value);
                return value;
            case TlvCodec_1.TlvType.EndOfContainer:
                throw new Error("Invalid template");
            case TlvCodec_1.TlvType.Array:
                return this.decodeArray(value, template);
            case TlvCodec_1.TlvType.List:
            case TlvCodec_1.TlvType.Structure:
                return this.decodeObject(value, template);
            default:
                throw new Error(`Unsupported element type ${type}`);
        }
    }
    static decodeArray(elements, { itemTemplate }) {
        return elements.map(element => this.decodeElement(element, itemTemplate));
    }
    static decodeObject(elements, { fieldTemplates }) {
        const result = {};
        for (var key in fieldTemplates) {
            const template = fieldTemplates[key];
            const { tag: expectedTag = TlvCodec_1.TlvTag.Anonymous } = template;
            const element = elements.find(({ tag }) => tag.equals(expectedTag));
            const value = this.decodeInternal(element, template);
            if (value === undefined) {
                if (!template.optional)
                    throw new Error(`Missing mandatory field ${key}`);
                continue;
            }
            result[key] = value;
        }
        return result;
    }
    static encode(value, template) {
        return TlvCodec_1.TlvCodec.encodeElement(this.encodeElement(value, template));
    }
    static encodeElement(value, template) {
        return this.encodeInternal(value, template);
    }
    static encodeInternal(value, template) {
        const { tlvType: type, tag = TlvCodec_1.TlvTag.Anonymous } = template;
        if (value === undefined)
            return undefined;
        switch (type) {
            case undefined:
                return Object.assign(Object.assign({}, value), { tag });
            case TlvCodec_1.TlvType.Boolean:
            case TlvCodec_1.TlvType.String:
            case TlvCodec_1.TlvType.ByteString:
            case TlvCodec_1.TlvType.SignedInt:
            case TlvCodec_1.TlvType.UnsignedInt:
            case TlvCodec_1.TlvType.Float:
            case TlvCodec_1.TlvType.Double:
            case TlvCodec_1.TlvType.Null:
                return { tag, type, value };
            case TlvCodec_1.TlvType.EndOfContainer:
                throw new Error("Invalid template");
            case TlvCodec_1.TlvType.Array:
                return { tag, type, value: this.encodeArray(value, template) };
            case TlvCodec_1.TlvType.List:
            case TlvCodec_1.TlvType.Structure:
                return { tag, type, value: this.encodeObject(value, template) };
            default:
                throw new Error(`Unsupported element type ${type}`);
        }
    }
    static encodeArray(array, { itemTemplate }) {
        return array.map(item => this.encodeInternal(item, itemTemplate));
    }
    static encodeObject(structure, { fieldTemplates }) {
        const result = new Array();
        for (var key in fieldTemplates) {
            const template = fieldTemplates[key];
            const value = structure[key];
            if (value === undefined) {
                if (!template.optional)
                    throw new Error(`Missing mandatory field ${key}`);
                continue;
            }
            const element = this.encodeInternal(value, template);
            if (element === undefined)
                continue;
            result.push(element);
        }
        return result;
    }
}
exports.TlvObjectCodec = TlvObjectCodec;
