/// <reference types="node" />
import BN from "bn.js";
import crypto from "crypto";
export interface KeyPair {
    publicKey: Buffer;
    privateKey: Buffer;
}
export declare class Crypto {
    static encrypt(key: Buffer, data: Buffer, nonce: Buffer, aad?: Buffer): Buffer;
    static decrypt(key: Buffer, data: Buffer, nonce: Buffer, aad?: Buffer): Buffer;
    static getRandomData(length: number): Buffer;
    static getRandom(): Buffer;
    static getRandomUInt16(): number;
    static getRandomUInt32(): number;
    static getRandomBN(size: number, maxValue: BN): BN;
    static ecdhGeneratePublicKey(): {
        publicKey: Buffer;
        ecdh: crypto.ECDH;
    };
    static ecdhGeneratePublicKeyAndSecret(peerPublicKey: Buffer): {
        publicKey: Buffer;
        sharedSecret: Buffer;
    };
    static ecdhGenerateSecret(peerPublicKey: Buffer, ecdh: crypto.ECDH): Buffer;
    static hash(data: Buffer | Buffer[]): Buffer;
    static pbkdf2(secret: Buffer, salt: Buffer, iteration: number, keyLength: number): Promise<Buffer>;
    static hkdf(secret: Buffer, salt: Buffer, info: Buffer, length?: number): Promise<Buffer>;
    static hmac(key: Buffer, data: Buffer): Buffer;
    static sign(privateKey: Buffer, data: Buffer | Buffer[], dsaEncoding?: ("ieee-p1363" | "der")): Buffer;
    static verify(publicKey: Buffer, data: Buffer, signature: Buffer, dsaEncoding?: ("ieee-p1363" | "der")): void;
    static createKeyPair(): KeyPair;
}
