/// <reference types="node" />
import { ResumptionRecord } from "./session/SessionManager";
import { NetInterface } from "../net/NetInterface";
import { InteractionClient } from "./interaction/InteractionClient";
import { Scanner } from "./common/Scanner";
import { Fabric } from "./fabric/Fabric";
export declare class MatterController {
    private readonly scanner;
    private readonly netInterface;
    private readonly certificateManager;
    private readonly fabric;
    static create(scanner: Scanner, netInterface: NetInterface): Promise<MatterController>;
    private readonly sessionManager;
    private readonly channelManager;
    private readonly exchangeManager;
    private readonly paseClient;
    private readonly caseClient;
    constructor(scanner: Scanner, netInterface: NetInterface, certificateManager: RootCertificateManager, fabric: Fabric);
    commission(commissionAddress: string, commissionPort: number, discriminator: number, setupPin: number): Promise<void>;
    connect(nodeId: bigint): Promise<InteractionClient>;
    private ensureSuccess;
    getNextAvailableSessionId(): number;
    createSecureSession(sessionId: number, fabric: Fabric | undefined, peerNodeId: bigint, peerSessionId: number, sharedSecret: Buffer, salt: Buffer, isInitiator: boolean, isResumption: boolean, idleRetransTimeoutMs?: number, activeRetransTimeoutMs?: number): Promise<import("./session/SecureSession").SecureSession<this>>;
    getResumptionRecord(resumptionId: Buffer): ResumptionRecord | undefined;
    findResumptionRecordByNodeId(nodeId: bigint): ResumptionRecord | undefined;
    saveResumptionRecord(resumptionRecord: ResumptionRecord): void;
    close(): void;
}
declare class RootCertificateManager {
    private readonly rootCertId;
    private readonly rootKeyPair;
    private readonly rootKeyIdentifier;
    private readonly rootCertBytes;
    private nextCertificateId;
    getRootCert(): Buffer;
    private generateRootCert;
    generateNoc(publicKey: Buffer, fabricId: bigint, nodeId: bigint): Buffer;
}
export {};
