/// <reference types="node" />
import { ResumptionRecord } from "./session/SessionManager";
import { Session } from "./session/Session";
import { Fabric } from "./fabric/Fabric";
import { NetInterface } from "../net/NetInterface";
import { Channel } from "../net/Channel";
import { ProtocolHandler } from "./common/ProtocolHandler";
import { Broadcaster } from "./common/Broadcaster";
import { Scanner } from "./common/Scanner";
export declare class MatterDevice {
    private readonly deviceName;
    private readonly deviceType;
    private readonly vendorId;
    private readonly productId;
    private readonly discriminator;
    private readonly scanners;
    private readonly broadcasters;
    private readonly netInterfaces;
    private readonly fabricManager;
    private readonly sessionManager;
    private readonly channelManager;
    private readonly exchangeManager;
    constructor(deviceName: string, deviceType: number, vendorId: number, productId: number, discriminator: number);
    addScanner(scanner: Scanner): this;
    addBroadcaster(broadcaster: Broadcaster): this;
    addNetInterface(netInterface: NetInterface): this;
    addProtocolHandler(protocol: ProtocolHandler<MatterDevice>): this;
    start(): void;
    getNextAvailableSessionId(): number;
    createSecureSession(sessionId: number, fabric: Fabric | undefined, peerNodeId: bigint, peerSessionId: number, sharedSecret: Buffer, salt: Buffer, isInitiator: boolean, isResumption: boolean, idleRetransTimeoutMs?: number, activeRetransTimeoutMs?: number): Promise<import("./session/SecureSession").SecureSession<this>>;
    findFabricFromDestinationId(destinationId: Buffer, peerRandom: Buffer): Fabric;
    setFabric(fabric: Fabric): void;
    initiateExchange(fabric: Fabric, nodeId: bigint, protocolId: number): import("./common/MessageExchange").MessageExchange<MatterDevice>;
    findResumptionRecordById(resumptionId: Buffer): ResumptionRecord | undefined;
    saveResumptionRecord(resumptionRecord: ResumptionRecord): void;
    armFailSafe(): void;
    getFabricBuilder(): import("./fabric/Fabric").FabricBuilder;
    completeCommission(): void;
    findDevice(fabric: Fabric, nodeId: bigint): Promise<undefined | {
        session: Session<MatterDevice>;
        channel: Channel<Buffer>;
    }>;
    stop(): void;
}
