import { Template } from "../../codec/TlvObjectCodec";
export interface Attribute<T> {
    id: number;
    template: Template<T>;
    optional: boolean;
    writable: boolean;
    conformanceValue?: T;
}
export interface OptionalAttribute<T> extends Attribute<T> {
    optional: true;
}
export interface WritableAttribute<T> extends Attribute<T> {
    writable: true;
}
export interface OptionalWritableAttribute<T> extends OptionalAttribute<T> {
    writable: true;
}
export declare type AttributeJsType<T extends Attribute<any>> = T extends Attribute<infer JsType> ? JsType : never;
export declare const Attribute: <T>(id: number, template: Template<T>, conformanceValue?: T | undefined) => Attribute<T>;
export declare const OptionalAttribute: <T>(id: number, template: Template<T>, conformanceValue?: T | undefined) => OptionalAttribute<T>;
export declare const WritableAttribute: <T>(id: number, template: Template<T>, conformanceValue?: T | undefined) => WritableAttribute<T>;
export declare const OptionalWritableAttribute: <T>(id: number, template: Template<T>, conformanceValue?: T | undefined) => WritableAttribute<T>;
export declare const NoArgumentsT: Template<import("../../util/Type").Merge<{}, {}, {} & {}>>;
export declare const NoResponseT: Template<void>;
export interface Command<RequestT, ResponseT> {
    optional: boolean;
    requestId: number;
    requestTemplate: Template<RequestT>;
    responseId: number;
    responseTemplate: Template<ResponseT>;
}
export interface OptionalCommand<RequestT, ResponseT> extends Command<RequestT, ResponseT> {
    optional: true;
}
export declare type ResponseType<T extends Command<any, any>> = T extends OptionalCommand<any, infer ResponseT> ? ResponseT | undefined : (T extends Command<any, infer ResponseT> ? ResponseT : never);
export declare type RequestType<T extends Command<any, any>> = T extends Command<infer RequestT, any> ? RequestT : never;
export declare const Command: <RequestT, ResponseT>(requestId: number, requestTemplate: Template<RequestT>, responseId: number, responseTemplate: Template<ResponseT>) => Command<RequestT, ResponseT>;
export declare const OptionalCommand: <RequestT, ResponseT>(requestId: number, requestTemplate: Template<RequestT>, responseId: number, responseTemplate: Template<ResponseT>) => OptionalCommand<RequestT, ResponseT>;
export interface Attributes {
    [key: string]: Attribute<any>;
}
export interface Commands {
    [key: string]: Command<any, any>;
}
export interface Cluster<C extends Commands, A extends Attributes> {
    id: number;
    name: string;
    commands: C;
    attributes: A;
}
export declare const Cluster: <C extends Commands, A extends Attributes>(id: number, name: string, attributes: A, commands: C) => Cluster<C, A>;
