/// <reference types="node" />
import { Attribute, Cluster, Command } from "./Cluster";
import { Template } from "../../codec/TlvObjectCodec";
export declare const enum CertificateType {
    DeviceAttestation = 1,
    ProductAttestationIntermediate = 2
}
export declare const CertificateTypeT: Template<CertificateType>;
export declare const AttestationRequestT: Template<import("../../util/Type").Merge<{
    attestationNonce: Buffer;
}, {}, {
    attestationNonce: Buffer;
} & {}>>;
export declare const AttestationResponseT: Template<import("../../util/Type").Merge<{
    elements: Buffer;
    signature: Buffer;
}, {}, {
    elements: Buffer;
    signature: Buffer;
} & {}>>;
export declare const CertSigningRequestRequestT: Template<import("../../util/Type").Merge<{
    certSigningRequestNonce: Buffer;
}, {
    isForUpdateNOC?: boolean | undefined;
}, {
    certSigningRequestNonce: Buffer;
} & {
    isForUpdateNOC?: boolean | undefined;
}>>;
export declare const CertSigningRequestResponseT: Template<import("../../util/Type").Merge<{
    elements: Buffer;
    signature: Buffer;
}, {}, {
    elements: Buffer;
    signature: Buffer;
} & {}>>;
export declare const CertChainRequestT: Template<import("../../util/Type").Merge<{
    type: CertificateType;
}, {}, {
    type: CertificateType;
} & {}>>;
export declare const CertChainResponseT: Template<import("../../util/Type").Merge<{
    certificate: Buffer;
}, {}, {
    certificate: Buffer;
} & {}>>;
export declare const AddNocRequestT: Template<import("../../util/Type").Merge<{
    operationalCert: Buffer;
    identityProtectionKey: Buffer;
    caseAdminNode: bigint;
    adminVendorId: number;
}, {
    intermediateCaCert?: Buffer | undefined;
}, {
    operationalCert: Buffer;
    identityProtectionKey: Buffer;
    caseAdminNode: bigint;
    adminVendorId: number;
} & {
    intermediateCaCert?: Buffer | undefined;
}>>;
export declare const UpdateNocRequestT: Template<import("../../util/Type").Merge<{
    operationalCert: Buffer;
}, {
    intermediateCaCert?: Buffer | undefined;
}, {
    operationalCert: Buffer;
} & {
    intermediateCaCert?: Buffer | undefined;
}>>;
export declare const AddTrustedRootCertificateRequestT: Template<import("../../util/Type").Merge<{
    certificate: Buffer;
}, {}, {
    certificate: Buffer;
} & {}>>;
export declare const enum OperationalCertStatus {
    Success = 0,
    InvalidPublicKey = 1,
    InvalidNodeOpId = 2,
    InvalidOperationalCert = 3,
    MissingCsr = 4,
    TableFull = 5,
    InvalidAdminSubject = 6,
    FabricConflict = 9,
    LabelConflict = 10,
    InvalidFabricIndex = 11
}
export declare const OperationalCertificateStatusResponseT: Template<import("../../util/Type").Merge<{
    status: OperationalCertStatus;
}, {
    debugText?: string | undefined;
    fabricIndex?: number | undefined;
}, {
    status: OperationalCertStatus;
} & {
    debugText?: string | undefined;
    fabricIndex?: number | undefined;
}>>;
export declare const AttestationT: Template<import("../../util/Type").Merge<{
    attestationNonce: Buffer;
    declaration: Buffer;
    timestamp: number;
}, {
    firmwareInfo?: Buffer | undefined;
}, {
    attestationNonce: Buffer;
    declaration: Buffer;
    timestamp: number;
} & {
    firmwareInfo?: Buffer | undefined;
}>>;
export declare const CertSigningRequestT: Template<import("../../util/Type").Merge<{
    certSigningRequestNonce: Buffer;
    certSigningRequest: Buffer;
}, {
    vendorReserved1?: Buffer | undefined;
    vendorReserved2?: Buffer | undefined;
    vendorReserved3?: Buffer | undefined;
}, {
    certSigningRequestNonce: Buffer;
    certSigningRequest: Buffer;
} & {
    vendorReserved1?: Buffer | undefined;
    vendorReserved2?: Buffer | undefined;
    vendorReserved3?: Buffer | undefined;
}>>;
export declare const UpdateFabricLabelRequestT: Template<import("../../util/Type").Merge<{
    fabricIndex: number;
}, {}, {
    fabricIndex: number;
} & {}>>;
export declare const RemoveFabricRequestT: Template<import("../../util/Type").Merge<{
    fabricIndex: number;
}, {}, {
    fabricIndex: number;
} & {}>>;
export declare const OperationalCredentialsCluster: Cluster<{
    requestAttestation: Command<import("../../util/Type").Merge<{
        attestationNonce: Buffer;
    }, {}, {
        attestationNonce: Buffer;
    } & {}>, import("../../util/Type").Merge<{
        elements: Buffer;
        signature: Buffer;
    }, {}, {
        elements: Buffer;
        signature: Buffer;
    } & {}>>;
    requestCertChain: Command<import("../../util/Type").Merge<{
        type: CertificateType;
    }, {}, {
        type: CertificateType;
    } & {}>, import("../../util/Type").Merge<{
        certificate: Buffer;
    }, {}, {
        certificate: Buffer;
    } & {}>>;
    requestCertSigning: Command<import("../../util/Type").Merge<{
        certSigningRequestNonce: Buffer;
    }, {
        isForUpdateNOC?: boolean | undefined;
    }, {
        certSigningRequestNonce: Buffer;
    } & {
        isForUpdateNOC?: boolean | undefined;
    }>, import("../../util/Type").Merge<{
        elements: Buffer;
        signature: Buffer;
    }, {}, {
        elements: Buffer;
        signature: Buffer;
    } & {}>>;
    addOperationalCert: Command<import("../../util/Type").Merge<{
        operationalCert: Buffer;
        identityProtectionKey: Buffer;
        caseAdminNode: bigint;
        adminVendorId: number;
    }, {
        intermediateCaCert?: Buffer | undefined;
    }, {
        operationalCert: Buffer;
        identityProtectionKey: Buffer;
        caseAdminNode: bigint;
        adminVendorId: number;
    } & {
        intermediateCaCert?: Buffer | undefined;
    }>, import("../../util/Type").Merge<{
        status: OperationalCertStatus;
    }, {
        debugText?: string | undefined;
        fabricIndex?: number | undefined;
    }, {
        status: OperationalCertStatus;
    } & {
        debugText?: string | undefined;
        fabricIndex?: number | undefined;
    }>>;
    updateOperationalCert: Command<import("../../util/Type").Merge<{
        operationalCert: Buffer;
    }, {
        intermediateCaCert?: Buffer | undefined;
    }, {
        operationalCert: Buffer;
    } & {
        intermediateCaCert?: Buffer | undefined;
    }>, import("../../util/Type").Merge<{
        status: OperationalCertStatus;
    }, {
        debugText?: string | undefined;
        fabricIndex?: number | undefined;
    }, {
        status: OperationalCertStatus;
    } & {
        debugText?: string | undefined;
        fabricIndex?: number | undefined;
    }>>;
    updateFabricLabel: Command<import("../../util/Type").Merge<{
        fabricIndex: number;
    }, {}, {
        fabricIndex: number;
    } & {}>, import("../../util/Type").Merge<{
        status: OperationalCertStatus;
    }, {
        debugText?: string | undefined;
        fabricIndex?: number | undefined;
    }, {
        status: OperationalCertStatus;
    } & {
        debugText?: string | undefined;
        fabricIndex?: number | undefined;
    }>>;
    removeFabric: Command<import("../../util/Type").Merge<{
        fabricIndex: number;
    }, {}, {
        fabricIndex: number;
    } & {}>, import("../../util/Type").Merge<{
        status: OperationalCertStatus;
    }, {
        debugText?: string | undefined;
        fabricIndex?: number | undefined;
    }, {
        status: OperationalCertStatus;
    } & {
        debugText?: string | undefined;
        fabricIndex?: number | undefined;
    }>>;
    addRootCert: Command<import("../../util/Type").Merge<{
        certificate: Buffer;
    }, {}, {
        certificate: Buffer;
    } & {}>, void>;
}, {
    nocs: Attribute<import("../../util/Type").Merge<{
        noc: Buffer;
        icac: Buffer;
    }, {}, {
        noc: Buffer;
        icac: Buffer;
    } & {}>[]>;
    fabrics: Attribute<import("../../util/Type").Merge<{
        label: string;
        vendorId: number;
        rootPublicKey: Buffer;
        fabricID: number;
        nodeID: number;
    }, {}, {
        label: string;
        vendorId: number;
        rootPublicKey: Buffer;
        fabricID: number;
        nodeID: number;
    } & {}>[]>;
    supportedFabrics: Attribute<number>;
    commissionedFabrics: Attribute<number>;
    trustedRootCertificates: Attribute<Buffer[]>;
    currentFabricIndex: Attribute<number>;
}>;
