"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AttributeServer = void 0;
class AttributeServer {
    constructor(id, name, template, defaultValue) {
        this.id = id;
        this.name = name;
        this.template = template;
        this.version = 0;
        this.matterListeners = new Array();
        this.listeners = new Array();
        this.value = defaultValue;
    }
    set(value) {
        const oldValue = this.value;
        if (value !== oldValue) {
            this.version++;
            this.value = value;
            this.matterListeners.forEach(listener => listener(value, this.version));
        }
        this.listeners.forEach(listener => listener(value, oldValue));
    }
    get() {
        return this.value;
    }
    getWithVersion() {
        return { version: this.version, value: this.value };
    }
    addMatterListener(listener) {
        this.matterListeners.push(listener);
    }
    removeMatterListener(listener) {
        this.matterListeners.splice(this.matterListeners.findIndex(item => item === listener), 1);
    }
    addListener(listener) {
        this.listeners.push(listener);
    }
    removeListener(listener) {
        this.listeners.splice(this.listeners.findIndex(item => item === listener), 1);
    }
}
exports.AttributeServer = AttributeServer;
