import { Merge } from "../../../util/Type";
import { MatterDevice } from "../../MatterDevice";
import { Session } from "../../session/Session";
import { Cluster, Command, Commands, AttributeJsType, Attributes, Attribute, OptionalAttribute, OptionalCommand } from "../Cluster";
import { AttributeServer } from "./AttributeServer";
declare type MandatoryAttributeNames<A extends Attributes> = {
    [K in keyof A]: A[K] extends OptionalAttribute<any> ? never : K;
}[keyof A];
declare type OptionalAttributeNames<A extends Attributes> = {
    [K in keyof A]: A[K] extends OptionalAttribute<any> ? K : never;
}[keyof A];
export declare type AttributeServers<A extends Attributes> = {
    [P in MandatoryAttributeNames<A>]: AttributeServer<AttributeJsType<A[P]>>;
};
export declare type AttributeInitialValues<A extends Attributes> = Merge<{
    [P in MandatoryAttributeNames<A>]: AttributeJsType<A[P]>;
}, {
    [P in OptionalAttributeNames<A>]?: AttributeJsType<A[P]>;
}>;
declare type MandatoryCommandNames<C extends Commands> = {
    [K in keyof C]: C[K] extends OptionalCommand<any, any> ? never : K;
}[keyof C];
declare type OptionalCommandNames<C extends Commands> = {
    [K in keyof C]: C[K] extends OptionalCommand<any, any> ? K : never;
}[keyof C];
declare type CommandHandler<C extends Command<any, any>, A extends AttributeServers<any>> = C extends Command<infer RequestT, infer ResponseT> ? (args: {
    request: RequestT;
    attributes: A;
    session: Session<MatterDevice>;
}) => Promise<ResponseT> : never;
declare type CommandHandlers<T extends Commands, A extends AttributeServers<any>> = Merge<{
    [P in MandatoryCommandNames<T>]: CommandHandler<T[P], A>;
}, {
    [P in OptionalCommandNames<T>]?: CommandHandler<T[P], A>;
}>;
export declare type ClusterServerHandlers<C extends Cluster<any, any>> = CommandHandlers<C["commands"], AttributeServers<C["attributes"]>>;
declare type OptionalAttributeConf<T extends Attributes> = {
    [K in OptionalAttributeNames<T>]?: true;
};
declare type MakeAttributesMandatory<T extends Attributes, C extends OptionalAttributeConf<T>> = Merge<Omit<T, keyof C>, {
    [K in Extract<keyof T, keyof C>]: Attribute<AttributeJsType<T[K]>>;
}>;
declare const MakeAttributesMandatory: <T extends Attributes, C extends OptionalAttributeConf<T>>(attributes: T, conf: C) => Merge<Omit<T, keyof C>, { [K in Extract<keyof T, keyof C>]: Attribute<AttributeJsType<T[K]>>; }, Omit<T, keyof C> & { [K in Extract<keyof T, keyof C>]: Attribute<AttributeJsType<T[K]>>; }>;
declare type UseOptionalAttributes<C extends Cluster<any, any>, A extends OptionalAttributeConf<C["attributes"]>> = Cluster<C["commands"], MakeAttributesMandatory<C["attributes"], A>>;
export declare const UseOptionalAttributes: <C extends Cluster<any, any>, A extends OptionalAttributeConf<C["attributes"]>>(cluster: C, conf: A) => UseOptionalAttributes<C, A>;
export {};
