"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OperationalCredentialsClusterHandler = void 0;
const TlvObjectCodec_1 = require("../../../codec/TlvObjectCodec");
const Crypto_1 = require("../../../crypto/Crypto");
const OperationalCredentialsCluster_1 = require("../OperationalCredentialsCluster");
function signWithDeviceKey(conf, session, data) {
    return Crypto_1.Crypto.sign(conf.devicePrivateKey, [data, session.getAttestationChallengeKey()]);
}
const OperationalCredentialsClusterHandler = (conf) => ({
    requestAttestation: ({ request: { attestationNonce }, session }) => __awaiter(void 0, void 0, void 0, function* () {
        const elements = TlvObjectCodec_1.TlvObjectCodec.encode({ declaration: conf.certificateDeclaration, attestationNonce, timestamp: 0 }, OperationalCredentialsCluster_1.AttestationT);
        return { elements: elements, signature: signWithDeviceKey(conf, session, elements) };
    }),
    requestCertSigning: ({ request: { certSigningRequestNonce }, session }) => __awaiter(void 0, void 0, void 0, function* () {
        const certSigningRequest = session.getContext().getFabricBuilder().createCertificateSigningRequest();
        const elements = TlvObjectCodec_1.TlvObjectCodec.encode({ certSigningRequest, certSigningRequestNonce }, OperationalCredentialsCluster_1.CertSigningRequestT);
        return { elements, signature: signWithDeviceKey(conf, session, elements) };
    }),
    requestCertChain: ({ request: { type } }) => __awaiter(void 0, void 0, void 0, function* () {
        switch (type) {
            case 1:
                return { certificate: conf.deviceCertificate };
            case 2:
                return { certificate: conf.deviceIntermediateCertificate };
            default:
                throw new Error(`Unsupported certificate type: ${type}`);
        }
    }),
    addOperationalCert: ({ request: { operationalCert, intermediateCaCert, identityProtectionKey, caseAdminNode, adminVendorId }, session }) => __awaiter(void 0, void 0, void 0, function* () {
        const fabricBuilder = session.getContext().getFabricBuilder();
        fabricBuilder.setOperationalCert(operationalCert);
        if (intermediateCaCert && intermediateCaCert.length > 0)
            fabricBuilder.setIntermediateCACert(intermediateCaCert);
        fabricBuilder.setVendorId(adminVendorId);
        fabricBuilder.setIdentityProtectionKey(identityProtectionKey);
        const fabric = yield fabricBuilder.build();
        session.getContext().setFabric(fabric);
        return { status: 0 };
    }),
    updateOperationalCert: ({ request: { operationalCert, intermediateCaCert, }, session }) => __awaiter(void 0, void 0, void 0, function* () {
        throw new Error("Not implemented");
        return { status: 0 };
    }),
    updateFabricLabel: ({ request: { fabricIndex } }) => __awaiter(void 0, void 0, void 0, function* () {
        throw new Error("Not implemented");
        return { status: 0 };
    }),
    removeFabric: ({ request: { fabricIndex } }) => __awaiter(void 0, void 0, void 0, function* () {
        throw new Error("Not implemented");
        return { status: 0 };
    }),
    addRootCert: ({ request: { certificate }, session }) => __awaiter(void 0, void 0, void 0, function* () {
        session.getContext().getFabricBuilder().setRootCert(certificate);
    }),
});
exports.OperationalCredentialsClusterHandler = OperationalCredentialsClusterHandler;
