"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChannelManager = void 0;
const ExchangeManager_1 = require("./ExchangeManager");
class ChannelManager {
    constructor() {
        this.channels = new Map();
    }
    setChannel(fabric, nodeId, channel) {
        this.channels.set(`${fabric.id}/${nodeId}`, channel);
    }
    getChannel(fabric, nodeId) {
        const result = this.channels.get(`${fabric.id}/${nodeId}`);
        if (result === undefined)
            throw new Error(`Can't find find a channel to node ${nodeId}`);
        return result;
    }
    getOrCreateChannel(bufferChannel, session) {
        if (!session.isSecure())
            return new ExchangeManager_1.MessageChannel(bufferChannel, session);
        const secureSession = session;
        const fabric = secureSession.getFabric();
        const nodeId = secureSession.getPeerNodeId();
        if (fabric === undefined)
            return new ExchangeManager_1.MessageChannel(bufferChannel, session);
        let result = this.channels.get(`${fabric.id}/${nodeId}`);
        if (result === undefined || result.session.getId() !== session.getId()) {
            result = new ExchangeManager_1.MessageChannel(bufferChannel, session);
            this.setChannel(fabric, nodeId, result);
        }
        return result;
    }
}
exports.ChannelManager = ChannelManager;
