/// <reference types="node" />
import { KeyPair } from "../../crypto/Crypto";
export declare class Fabric {
    readonly id: bigint;
    readonly nodeId: bigint;
    readonly operationalId: Buffer;
    readonly rootPublicKey: Buffer;
    private readonly keyPair;
    private readonly vendorId;
    private readonly rootCert;
    readonly identityProtectionKey: Buffer;
    readonly operationalIdentityProtectionKey: Buffer;
    readonly intermediateCACert: Buffer | undefined;
    readonly operationalCert: Buffer;
    constructor(id: bigint, nodeId: bigint, operationalId: Buffer, rootPublicKey: Buffer, keyPair: KeyPair, vendorId: number, rootCert: Buffer, identityProtectionKey: Buffer, operationalIdentityProtectionKey: Buffer, intermediateCACert: Buffer | undefined, operationalCert: Buffer);
    getPublicKey(): Buffer;
    sign(data: Buffer): Buffer;
    verifyCredentials(operationalCert: Buffer, intermediateCACert: Buffer | undefined): void;
    getDestinationId(nodeId: bigint, random: Buffer): Buffer;
}
export declare class FabricBuilder {
    private keyPair;
    private vendorId?;
    private rootCert?;
    private intermediateCACert?;
    private operationalCert?;
    private fabricId?;
    private nodeId?;
    private rootPublicKey?;
    private identityProtectionKey?;
    getPublicKey(): Buffer;
    createCertificateSigningRequest(): Buffer;
    setRootCert(rootCert: Buffer): this;
    setOperationalCert(operationalCert: Buffer): this;
    setIntermediateCACert(certificate: Buffer): this;
    setVendorId(vendorId: number): this;
    setIdentityProtectionKey(key: Buffer): this;
    build(): Promise<Fabric>;
}
