"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.InteractionClient = exports.SubscriptionClient = exports.ClusterClient = void 0;
const TlvObjectCodec_1 = require("../../codec/TlvObjectCodec");
const String_1 = require("../../util/String");
const Cluster_1 = require("../cluster/Cluster");
const InteractionMessenger_1 = require("./InteractionMessenger");
const InteractionServer_1 = require("./InteractionServer");
function ClusterClient(interactionClient, endpointId, clusterDef) {
    const result = {};
    const { id: clusterId, commands, attributes } = clusterDef;
    for (const attributeName in attributes) {
        const attribute = attributes[attributeName];
        const captilizedAttributeName = (0, String_1.capitalize)(attributeName);
        result[`get${captilizedAttributeName}`] = () => __awaiter(this, void 0, void 0, function* () { return interactionClient.get(endpointId, clusterId, attribute); });
        result[`set${captilizedAttributeName}`] = (value) => __awaiter(this, void 0, void 0, function* () { return interactionClient.set(endpointId, clusterId, attribute, value); });
        result[`subscribe${captilizedAttributeName}`] = (listener, minIntervalS, maxIntervalS) => __awaiter(this, void 0, void 0, function* () { return interactionClient.subscribe(endpointId, clusterId, attribute, listener, minIntervalS, maxIntervalS); });
    }
    for (const commandName in commands) {
        const { requestId, requestTemplate, responseId, responseTemplate, optional } = commands[commandName];
        result[commandName] = (request) => __awaiter(this, void 0, void 0, function* () { return interactionClient.invoke(endpointId, clusterId, request, requestId, requestTemplate, responseId, responseTemplate, optional); });
    }
    return result;
}
exports.ClusterClient = ClusterClient;
class SubscriptionClient {
    constructor(subscriptionListeners) {
        this.subscriptionListeners = subscriptionListeners;
    }
    getId() {
        return InteractionServer_1.INTERACTION_PROTOCOL_ID;
    }
    onNewExchange(exchange) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const messenger = new InteractionMessenger_1.InteractionClientMessenger(exchange);
            const dataReport = yield messenger.readDataReport();
            yield messenger.sendStatus(0);
            const subscriptionId = dataReport.subscriptionId;
            if (subscriptionId === undefined)
                return;
            (_a = this.subscriptionListeners.get(subscriptionId)) === null || _a === void 0 ? void 0 : _a(dataReport);
        });
    }
}
exports.SubscriptionClient = SubscriptionClient;
class InteractionClient {
    constructor(exchangeManager, channel) {
        this.exchangeManager = exchangeManager;
        this.channel = channel;
        this.subscriptionListeners = new Map();
        this.exchangeManager.addProtocolHandler(new SubscriptionClient(this.subscriptionListeners));
    }
    get(endpointId, clusterId, { id, template, optional, conformanceValue }) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.withMessenger((messenger) => __awaiter(this, void 0, void 0, function* () {
                const response = yield messenger.sendReadRequest({
                    attributes: [{ endpointId, clusterId, id }],
                    interactionModelRevision: 1,
                    isFabricFiltered: true,
                });
                const value = response.values.map(({ value }) => value).find(({ path }) => endpointId === path.endpointId && clusterId === path.clusterId && id === path.id);
                if (value === undefined) {
                    if (optional)
                        return undefined;
                    if (conformanceValue === undefined)
                        throw new Error(`Attribute ${endpointId}/${clusterId}/${id} not found`);
                    return conformanceValue;
                }
                return TlvObjectCodec_1.TlvObjectCodec.decodeElement(value.value, template);
            }));
        });
    }
    set(endpointId, clusterId, { id, template, conformanceValue }, value) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error("not implemented");
        });
    }
    subscribe(endpointId, clusterId, { id, template, conformanceValue }, listener, minIntervalFloorSeconds, maxIntervalCeilingSeconds) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.withMessenger((messenger) => __awaiter(this, void 0, void 0, function* () {
                const { subscriptionId } = yield messenger.sendSubscribeRequest({
                    attributeRequests: [{ endpointId, clusterId, id }],
                    keepSubscriptions: true,
                    minIntervalFloorSeconds,
                    maxIntervalCeilingSeconds,
                    isFabricFiltered: true,
                });
                this.subscriptionListeners.set(subscriptionId, (dataReport) => {
                    const value = dataReport.values.map(({ value }) => value).find(({ path }) => endpointId === path.endpointId && clusterId === path.clusterId && id === path.id);
                    if (value === undefined)
                        return;
                    listener(TlvObjectCodec_1.TlvObjectCodec.decodeElement(value.value, template), value.version);
                });
                return;
            }));
        });
    }
    invoke(endpointId, clusterId, request, id, requestTemplate, responseId, responseTemplate, optional) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.withMessenger((messenger) => __awaiter(this, void 0, void 0, function* () {
                const { responses } = yield messenger.sendInvokeCommand({
                    invokes: [
                        { path: { endpointId, clusterId, id }, args: TlvObjectCodec_1.TlvObjectCodec.encodeElement(request, requestTemplate) }
                    ],
                    timedRequest: false,
                    suppressResponse: false,
                });
                if (responses.length === 0)
                    throw new Error("No response received");
                const { response, result } = responses[0];
                if (result !== undefined) {
                    const resultCode = result.result.code;
                    if (resultCode !== 0)
                        throw new Error(`Received non-success result: ${resultCode}`);
                    if (responseTemplate !== Cluster_1.NoResponseT)
                        throw new Error("A response was expected for this command");
                    return undefined;
                }
                if (response !== undefined) {
                    return TlvObjectCodec_1.TlvObjectCodec.decodeElement(response.response, responseTemplate);
                }
                if (optional) {
                    return undefined;
                }
                throw new Error("Received invoke response with no result nor response");
            }));
        });
    }
    withMessenger(invoke) {
        return __awaiter(this, void 0, void 0, function* () {
            const messenger = new InteractionMessenger_1.InteractionClientMessenger(this.exchangeManager.initiateExchangeWithChannel(this.channel, InteractionServer_1.INTERACTION_PROTOCOL_ID));
            try {
                return yield invoke(messenger);
            }
            finally {
                messenger.close();
            }
        });
    }
}
exports.InteractionClient = InteractionClient;
