/// <reference types="node" />
import { JsType } from "../../codec/TlvObjectCodec";
import { MessageExchange } from "../common/MessageExchange";
import { MatterController } from "../MatterController";
import { MatterDevice } from "../MatterDevice";
import { InvokeRequestT, InvokeResponseT, ReadRequestT, DataReportT, SubscribeRequestT, SubscribeResponseT, StatusCode } from "./InteractionMessages";
export declare const enum MessageType {
    StatusResponse = 1,
    ReadRequest = 2,
    SubscribeRequest = 3,
    SubscribeResponse = 4,
    ReportData = 5,
    WriteRequest = 6,
    WriteResponse = 7,
    InvokeCommandRequest = 8,
    InvokeCommandResponse = 9,
    TimedRequest = 10
}
export declare type ReadRequest = JsType<typeof ReadRequestT>;
export declare type DataReport = JsType<typeof DataReportT>;
export declare type SubscribeRequest = JsType<typeof SubscribeRequestT>;
export declare type SubscribeResponse = JsType<typeof SubscribeResponseT>;
export declare type InvokeRequest = JsType<typeof InvokeRequestT>;
export declare type InvokeResponse = JsType<typeof InvokeResponseT>;
declare class InteractionMessenger<ContextT> {
    private readonly exchangeBase;
    constructor(exchangeBase: MessageExchange<ContextT>);
    sendStatus(status: StatusCode): Promise<void>;
    waitForSuccess(): Promise<void>;
    nextMessage(expectedMessageType?: number): Promise<import("../../codec/MessageCodec").Message>;
    close(): void;
    protected throwIfError(messageType: number, payload: Buffer): void;
}
export declare class InteractionServerMessenger extends InteractionMessenger<MatterDevice> {
    private readonly exchange;
    constructor(exchange: MessageExchange<MatterDevice>);
    handleRequest(handleReadRequest: (request: ReadRequest) => DataReport, handleSubscribeRequest: (request: SubscribeRequest) => SubscribeResponse | undefined, handleInvokeRequest: (request: InvokeRequest) => Promise<InvokeResponse>): Promise<void>;
    sendDataReport(dataReport: DataReport): Promise<void>;
}
export declare class InteractionClientMessenger extends InteractionMessenger<MatterController> {
    private readonly exchange;
    constructor(exchange: MessageExchange<MatterController>);
    sendReadRequest(readRequest: ReadRequest): Promise<import("../../util/Type").Merge<{
        values: import("../../util/Type").Merge<{
            value: import("../../util/Type").Merge<{
                value: any;
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                version: number;
            }, {}, {
                value: any;
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                version: number;
            } & {}>;
        }, {}, {
            value: import("../../util/Type").Merge<{
                value: any;
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                version: number;
            }, {}, {
                value: any;
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                version: number;
            } & {}>;
        } & {}>[];
        interactionModelRevision: number;
    }, {
        isFabricFiltered?: boolean | undefined;
        subscriptionId?: number | undefined;
    }, {
        values: import("../../util/Type").Merge<{
            value: import("../../util/Type").Merge<{
                value: any;
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                version: number;
            }, {}, {
                value: any;
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                version: number;
            } & {}>;
        }, {}, {
            value: import("../../util/Type").Merge<{
                value: any;
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                version: number;
            }, {}, {
                value: any;
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                version: number;
            } & {}>;
        } & {}>[];
        interactionModelRevision: number;
    } & {
        isFabricFiltered?: boolean | undefined;
        subscriptionId?: number | undefined;
    }>>;
    sendSubscribeRequest(subscribeRequest: SubscribeRequest): Promise<import("../../util/Type").Merge<{
        interactionModelRevision: number;
        subscriptionId: number;
        maxIntervalCeilingSeconds: number;
    }, {}, {
        interactionModelRevision: number;
        subscriptionId: number;
        maxIntervalCeilingSeconds: number;
    } & {}>>;
    sendInvokeCommand(invokeRequest: InvokeRequest): Promise<import("../../util/Type").Merge<{
        interactionModelRevision: number;
        suppressResponse: boolean;
        responses: import("../../util/Type").Merge<{}, {
            response?: import("../../util/Type").Merge<{
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                response: any;
            }, {}, {
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                response: any;
            } & {}> | undefined;
            result?: import("../../util/Type").Merge<{
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                result: import("../../util/Type").Merge<{
                    code: number;
                }, {}, {
                    code: number;
                } & {}>;
            }, {}, {
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                result: import("../../util/Type").Merge<{
                    code: number;
                }, {}, {
                    code: number;
                } & {}>;
            } & {}> | undefined;
        }, {} & {
            response?: import("../../util/Type").Merge<{
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                response: any;
            }, {}, {
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                response: any;
            } & {}> | undefined;
            result?: import("../../util/Type").Merge<{
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                result: import("../../util/Type").Merge<{
                    code: number;
                }, {}, {
                    code: number;
                } & {}>;
            }, {}, {
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                result: import("../../util/Type").Merge<{
                    code: number;
                }, {}, {
                    code: number;
                } & {}>;
            } & {}> | undefined;
        }>[];
    }, {}, {
        interactionModelRevision: number;
        suppressResponse: boolean;
        responses: import("../../util/Type").Merge<{}, {
            response?: import("../../util/Type").Merge<{
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                response: any;
            }, {}, {
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                response: any;
            } & {}> | undefined;
            result?: import("../../util/Type").Merge<{
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                result: import("../../util/Type").Merge<{
                    code: number;
                }, {}, {
                    code: number;
                } & {}>;
            }, {}, {
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                result: import("../../util/Type").Merge<{
                    code: number;
                }, {}, {
                    code: number;
                } & {}>;
            } & {}> | undefined;
        }, {} & {
            response?: import("../../util/Type").Merge<{
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                response: any;
            }, {}, {
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                response: any;
            } & {}> | undefined;
            result?: import("../../util/Type").Merge<{
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                result: import("../../util/Type").Merge<{
                    code: number;
                }, {}, {
                    code: number;
                } & {}>;
            }, {}, {
                path: import("../../util/Type").Merge<{
                    id: number;
                    endpointId: number;
                    clusterId: number;
                }, {}, {
                    id: number;
                    endpointId: number;
                    clusterId: number;
                } & {}>;
                result: import("../../util/Type").Merge<{
                    code: number;
                }, {}, {
                    code: number;
                } & {}>;
            } & {}> | undefined;
        }>[];
    } & {}>>;
    readDataReport(): Promise<DataReport>;
    private request;
}
export {};
