import { MatterDevice } from "../MatterDevice";
import { ProtocolHandler } from "../common/ProtocolHandler";
import { MessageExchange } from "../common/MessageExchange";
import { InvokeRequest, InvokeResponse, ReadRequest, DataReport, SubscribeRequest, SubscribeResponse } from "./InteractionMessenger";
import { CommandServer } from "../cluster/server/CommandServer";
import { AttributeServer } from "../cluster/server/AttributeServer";
import { Cluster } from "../cluster/Cluster";
import { AttributeServers, AttributeInitialValues, ClusterServerHandlers } from "../cluster/server/ClusterServer";
export declare const INTERACTION_PROTOCOL_ID = 1;
export declare class ClusterServer<ClusterT extends Cluster<any, any>> {
    readonly id: number;
    readonly attributes: AttributeServers<ClusterT["attributes"]>;
    readonly commands: CommandServer<any, any>[];
    constructor(clusterDef: ClusterT, attributesInitialValues: AttributeInitialValues<ClusterT["attributes"]>, handlers: ClusterServerHandlers<ClusterT>);
}
export interface Path {
    endpointId: number;
    clusterId: number;
    id: number;
}
export interface AttributeWithPath {
    path: Path;
    attribute: AttributeServer<any>;
}
export declare function pathToId({ endpointId, clusterId, id }: Path): string;
export declare class InteractionServer implements ProtocolHandler<MatterDevice> {
    private readonly attributes;
    private readonly attributePaths;
    private readonly commands;
    private readonly commandPaths;
    constructor();
    getId(): number;
    addEndpoint(endpointId: number, device: {
        name: string;
        code: number;
    }, clusters: ClusterServer<any>[]): this;
    onNewExchange(exchange: MessageExchange<MatterDevice>): Promise<void>;
    handleReadRequest(exchange: MessageExchange<MatterDevice>, { attributes: attributePaths }: ReadRequest): DataReport;
    handleSubscribeRequest(exchange: MessageExchange<MatterDevice>, { minIntervalFloorSeconds, maxIntervalCeilingSeconds, attributeRequests, keepSubscriptions }: SubscribeRequest): SubscribeResponse | undefined;
    handleInvokeRequest(exchange: MessageExchange<MatterDevice>, { invokes }: InvokeRequest): Promise<InvokeResponse>;
    private getAttributes;
}
