"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SubscriptionHandler = void 0;
const InteractionMessenger_1 = require("./InteractionMessenger");
const TlvObjectCodec_1 = require("../../codec/TlvObjectCodec");
const InteractionServer_1 = require("./InteractionServer");
const Time_1 = require("../../time/Time");
class SubscriptionHandler {
    constructor(subscriptionId, server, fabric, peerNodeId, attributes, minIntervalFloorMs, maxIntervalCeilingMs) {
        this.subscriptionId = subscriptionId;
        this.server = server;
        this.fabric = fabric;
        this.peerNodeId = peerNodeId;
        this.attributes = attributes;
        this.minIntervalFloorMs = minIntervalFloorMs;
        this.maxIntervalCeilingMs = maxIntervalCeilingMs;
        this.lastUpdateTimeMs = 0;
        this.listener = () => this.sendUpdate();
        attributes.forEach(({ attribute }) => attribute.addMatterListener(this.listener));
        this.updateTimer = Time_1.Time.getTimer(this.minIntervalFloorMs, () => this.sendUpdate()).start();
    }
    sendUpdate() {
        return __awaiter(this, void 0, void 0, function* () {
            const now = Date.now();
            const timeSinceLastUpdateMs = now - this.lastUpdateTimeMs;
            if (timeSinceLastUpdateMs < this.minIntervalFloorMs) {
                this.updateTimer.stop();
                this.updateTimer = Time_1.Time.getTimer(this.minIntervalFloorMs - timeSinceLastUpdateMs, () => this.sendUpdate()).start();
                return;
            }
            const values = this.attributes.map(({ attribute, path }) => ({ path, valueVersion: attribute.getWithVersion(), template: attribute.template }));
            yield this.sendUpdateMessage(values);
            this.lastUpdateTimeMs = now;
            this.updateTimer.stop();
            this.updateTimer = Time_1.Time.getTimer(this.maxIntervalCeilingMs, () => this.sendUpdate()).start();
        });
    }
    cancel() {
        this.attributes.forEach(({ attribute }) => attribute.removeMatterListener(this.listener));
        this.updateTimer.stop();
    }
    sendUpdateMessage(values) {
        return __awaiter(this, void 0, void 0, function* () {
            const exchange = this.server.initiateExchange(this.fabric, this.peerNodeId, InteractionServer_1.INTERACTION_PROTOCOL_ID);
            if (exchange === undefined)
                return;
            const messenger = new InteractionMessenger_1.InteractionServerMessenger(exchange);
            yield messenger.sendDataReport({
                subscriptionId: this.subscriptionId,
                interactionModelRevision: 1,
                values: values.map(({ path, template, valueVersion: { value, version } }) => ({
                    value: {
                        path,
                        version,
                        value: TlvObjectCodec_1.TlvObjectCodec.encodeElement(value, template),
                    },
                })),
            });
            yield messenger.waitForSuccess();
            messenger.close();
        });
    }
}
exports.SubscriptionHandler = SubscriptionHandler;
SubscriptionHandler.Builder = (server, fabric, peerNodeId, attributes, minIntervalFloorSeconds, maxIntervalCeilingSeconds) => (subscriptionId) => new SubscriptionHandler(subscriptionId, server, fabric, peerNodeId, attributes, minIntervalFloorSeconds * 1000, maxIntervalCeilingSeconds * 1000);
