/// <reference types="node" />
import { Message, Packet } from "../../codec/MessageCodec";
import { Fabric } from "../fabric/Fabric";
import { SubscriptionHandler } from "../interaction/SubscriptionHandler";
import { Session } from "./Session";
export declare class SecureSession<T> implements Session<T> {
    private readonly context;
    private readonly id;
    private readonly fabric;
    private readonly peerNodeId;
    private readonly peerSessionId;
    private readonly sharedSecret;
    private readonly decryptKey;
    private readonly encryptKey;
    private readonly attestationKey;
    private readonly idleRetransmissionTimeoutMs;
    private readonly activeRetransmissionTimeoutMs;
    private readonly retransmissionRetries;
    private nextSubscriptionId;
    private readonly subscriptions;
    static create<T>(context: T, id: number, fabric: Fabric | undefined, peerNodeId: bigint, peerSessionId: number, sharedSecret: Buffer, salt: Buffer, isInitiator: boolean, isResumption: boolean, idleRetransTimeoutMs?: number, activeRetransTimeoutMs?: number): Promise<SecureSession<T>>;
    constructor(context: T, id: number, fabric: Fabric | undefined, peerNodeId: bigint, peerSessionId: number, sharedSecret: Buffer, decryptKey: Buffer, encryptKey: Buffer, attestationKey: Buffer, idleRetransmissionTimeoutMs?: number, activeRetransmissionTimeoutMs?: number, retransmissionRetries?: number);
    isSecure(): boolean;
    decode({ header, bytes }: Packet): Message;
    encode(message: Message): Packet;
    getAttestationChallengeKey(): Buffer;
    getFabric(): Fabric | undefined;
    getName(): string;
    getMrpParameters(): {
        idleRetransmissionTimeoutMs: number;
        activeRetransmissionTimeoutMs: number;
        retransmissionRetries: number;
    };
    getContext(): T;
    getId(): number;
    getPeerSessionId(): number;
    getNodeId(): bigint;
    getPeerNodeId(): bigint;
    addSubscription(subscriptionBuilder: (subscriptionId: number) => SubscriptionHandler): number;
    clearSubscriptions(): void;
    private generateNonce;
}
