/// <reference types="node" />
import { Fabric } from "../fabric/Fabric";
import { SecureSession } from "./SecureSession";
import { Session } from "./Session";
import { UnsecureSession } from "./UnsecureSession";
export declare const UNDEFINED_NODE_ID: bigint;
export declare const UNICAST_UNSECURE_SESSION_ID = 0;
export interface ResumptionRecord {
    sharedSecret: Buffer;
    resumptionId: Buffer;
    fabric: Fabric;
    peerNodeId: bigint;
}
export declare class SessionManager<ContextT> {
    private readonly context;
    private readonly unsecureSession;
    private readonly sessions;
    private nextSessionId;
    private resumptionRecords;
    constructor(context: ContextT);
    createSecureSession(sessionId: number, fabric: Fabric | undefined, peerNodeId: bigint, peerSessionId: number, sharedSecret: Buffer, salt: Buffer, isInitiator: boolean, isResumption: boolean, idleRetransTimeoutMs?: number, activeRetransTimeoutMs?: number): Promise<SecureSession<ContextT>>;
    getNextAvailableSessionId(): number;
    getSession(sessionId: number): Session<ContextT> | undefined;
    getSessionForNode(fabric: Fabric, nodeId: bigint): Session<ContextT> | undefined;
    getUnsecureSession(): UnsecureSession<ContextT>;
    findResumptionRecordById(resumptionId: Buffer): ResumptionRecord | undefined;
    findResumptionRecordByNodeId(nodeId: bigint): ResumptionRecord | undefined;
    saveResumptionRecord(resumptionRecord: ResumptionRecord): void;
}
