/// <reference types="node" />
import { JsType } from "../../../codec/TlvObjectCodec";
import { MatterController } from "../../MatterController";
import { MatterDevice } from "../../MatterDevice";
import { CaseSigma1T, CaseSigma2ResumeT, CaseSigma2T, CaseSigma3T } from "./CaseMessages";
import { SecureChannelMessenger } from "./SecureChannelMessenger";
export declare class CaseServerMessenger extends SecureChannelMessenger<MatterDevice> {
    readSigma1(): Promise<{
        sigma1Bytes: Buffer;
        sigma1: import("../../../util/Type").Merge<{
            sessionId: number;
            random: Buffer;
            destinationId: Buffer;
            ecdhPublicKey: Buffer;
        }, {
            mrpParams?: import("../../../util/Type").Merge<{}, {
                idleRetransTimeoutMs?: number | undefined;
                activeRetransTimeoutMs?: number | undefined;
            }, {} & {
                idleRetransTimeoutMs?: number | undefined;
                activeRetransTimeoutMs?: number | undefined;
            }> | undefined;
            resumptionId?: Buffer | undefined;
            resumeMic?: Buffer | undefined;
        }, {
            sessionId: number;
            random: Buffer;
            destinationId: Buffer;
            ecdhPublicKey: Buffer;
        } & {
            mrpParams?: import("../../../util/Type").Merge<{}, {
                idleRetransTimeoutMs?: number | undefined;
                activeRetransTimeoutMs?: number | undefined;
            }, {} & {
                idleRetransTimeoutMs?: number | undefined;
                activeRetransTimeoutMs?: number | undefined;
            }> | undefined;
            resumptionId?: Buffer | undefined;
            resumeMic?: Buffer | undefined;
        }>;
    }>;
    sendSigma2(sigma2: JsType<typeof CaseSigma2T>): Promise<Buffer>;
    sendSigma2Resume(sigma2Resume: JsType<typeof CaseSigma2ResumeT>): Promise<Buffer>;
    readSigma3(): Promise<{
        sigma3Bytes: Buffer;
        sigma3: import("../../../util/Type").Merge<{
            encrypted: Buffer;
        }, {}, {
            encrypted: Buffer;
        } & {}>;
    }>;
}
export declare class CaseClientMessenger extends SecureChannelMessenger<MatterController> {
    sendSigma1(sigma1: JsType<typeof CaseSigma1T>): Promise<Buffer>;
    readSigma2(): Promise<{
        sigma2Bytes: Buffer;
        sigma2: import("../../../util/Type").Merge<{
            encrypted: Buffer;
            sessionId: number;
            random: Buffer;
            ecdhPublicKey: Buffer;
        }, {
            mrpParams?: import("../../../util/Type").Merge<{}, {
                idleRetransTimeoutMs?: number | undefined;
                activeRetransTimeoutMs?: number | undefined;
            }, {} & {
                idleRetransTimeoutMs?: number | undefined;
                activeRetransTimeoutMs?: number | undefined;
            }> | undefined;
        }, {
            encrypted: Buffer;
            sessionId: number;
            random: Buffer;
            ecdhPublicKey: Buffer;
        } & {
            mrpParams?: import("../../../util/Type").Merge<{}, {
                idleRetransTimeoutMs?: number | undefined;
                activeRetransTimeoutMs?: number | undefined;
            }, {} & {
                idleRetransTimeoutMs?: number | undefined;
                activeRetransTimeoutMs?: number | undefined;
            }> | undefined;
        }>;
        sigma2Resume?: undefined;
    } | {
        sigma2Resume: import("../../../util/Type").Merge<{
            sessionId: number;
            resumptionId: Buffer;
            resumeMic: Buffer;
        }, {}, {
            sessionId: number;
            resumptionId: Buffer;
            resumeMic: Buffer;
        } & {}>;
        sigma2Bytes?: undefined;
        sigma2?: undefined;
    }>;
    sendSigma3(sigma3: JsType<typeof CaseSigma3T>): Promise<Buffer>;
}
