/// <reference types="node" />
import { JsType } from "../../../codec/TlvObjectCodec";
import { MatterController } from "../../MatterController";
import { MatterDevice } from "../../MatterDevice";
import { PasePake1T, PasePake2T, PasePake3T, PbkdfParamRequestT, PbkdfParamResponseT } from "./PaseMessages";
import { SecureChannelMessenger } from "./SecureChannelMessenger";
export declare const DEFAULT_PASSCODE_ID = 0;
export declare const SPAKE_CONTEXT: Buffer;
declare type PbkdfParamRequest = JsType<typeof PbkdfParamRequestT>;
declare type PbkdfParamResponse = JsType<typeof PbkdfParamResponseT>;
declare type PasePake1 = JsType<typeof PasePake1T>;
declare type PasePake2 = JsType<typeof PasePake2T>;
declare type PasePake3 = JsType<typeof PasePake3T>;
export declare class PaseServerMessenger extends SecureChannelMessenger<MatterDevice> {
    readPbkdfParamRequest(): Promise<{
        requestPayload: Buffer;
        request: import("../../../util/Type").Merge<{
            sessionId: number;
            random: Buffer;
            passcodeId: number;
            hasPbkdfParameters: boolean;
        }, {
            mrpParameters?: import("../../../util/Type").Merge<{}, {
                idleRetransTimeoutMs?: number | undefined;
                activeRetransTimeoutMs?: number | undefined;
            }, {} & {
                idleRetransTimeoutMs?: number | undefined;
                activeRetransTimeoutMs?: number | undefined;
            }> | undefined;
        }, {
            sessionId: number;
            random: Buffer;
            passcodeId: number;
            hasPbkdfParameters: boolean;
        } & {
            mrpParameters?: import("../../../util/Type").Merge<{}, {
                idleRetransTimeoutMs?: number | undefined;
                activeRetransTimeoutMs?: number | undefined;
            }, {} & {
                idleRetransTimeoutMs?: number | undefined;
                activeRetransTimeoutMs?: number | undefined;
            }> | undefined;
        }>;
    }>;
    sendPbkdfParamResponse(response: PbkdfParamResponse): Promise<Buffer>;
    readPasePake1(): Promise<import("../../../util/Type").Merge<{
        x: Buffer;
    }, {}, {
        x: Buffer;
    } & {}>>;
    sendPasePake2(pasePake2: PasePake2): Promise<Buffer>;
    readPasePake3(): Promise<import("../../../util/Type").Merge<{
        verifier: Buffer;
    }, {}, {
        verifier: Buffer;
    } & {}>>;
}
export declare class PaseClientMessenger extends SecureChannelMessenger<MatterController> {
    sendPbkdfParamRequest(request: PbkdfParamRequest): Promise<Buffer>;
    readPbkdfParamResponse(): Promise<{
        responsePayload: Buffer;
        response: import("../../../util/Type").Merge<{
            sessionId: number;
            random: Buffer;
            peerRandom: Buffer;
        }, {
            mrpParameters?: import("../../../util/Type").Merge<{}, {
                idleRetransTimeoutMs?: number | undefined;
                activeRetransTimeoutMs?: number | undefined;
            }, {} & {
                idleRetransTimeoutMs?: number | undefined;
                activeRetransTimeoutMs?: number | undefined;
            }> | undefined;
            pbkdfParameters?: import("../../../util/Type").Merge<{
                iteration: number;
                salt: Buffer;
            }, {}, {
                iteration: number;
                salt: Buffer;
            } & {}> | undefined;
        }, {
            sessionId: number;
            random: Buffer;
            peerRandom: Buffer;
        } & {
            mrpParameters?: import("../../../util/Type").Merge<{}, {
                idleRetransTimeoutMs?: number | undefined;
                activeRetransTimeoutMs?: number | undefined;
            }, {} & {
                idleRetransTimeoutMs?: number | undefined;
                activeRetransTimeoutMs?: number | undefined;
            }> | undefined;
            pbkdfParameters?: import("../../../util/Type").Merge<{
                iteration: number;
                salt: Buffer;
            }, {}, {
                iteration: number;
                salt: Buffer;
            } & {}> | undefined;
        }>;
    }>;
    sendPasePake1(pasePake1: PasePake1): Promise<Buffer>;
    readPasePake2(): Promise<import("../../../util/Type").Merge<{
        y: Buffer;
        verifier: Buffer;
    }, {}, {
        y: Buffer;
        verifier: Buffer;
    } & {}>>;
    sendPasePake3(pasePake3: PasePake3): Promise<Buffer>;
}
export {};
