"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SecureChannelMessenger = void 0;
const TlvObjectCodec_1 = require("../../../codec/TlvObjectCodec");
const LEBufferReader_1 = require("../../../util/LEBufferReader");
const LEBufferWriter_1 = require("../../../util/LEBufferWriter");
const SecureChannelMessages_1 = require("./SecureChannelMessages");
class SecureChannelMessenger {
    constructor(exchange) {
        this.exchange = exchange;
    }
    nextMessage(expectedMessageType) {
        return __awaiter(this, void 0, void 0, function* () {
            const message = yield this.exchange.nextMessage();
            const messageType = message.payloadHeader.messageType;
            this.throwIfError(messageType, message.payload);
            if (expectedMessageType !== undefined && messageType !== expectedMessageType)
                throw new Error(`Received unexpected message type: ${messageType}, expected: ${expectedMessageType}`);
            return message;
        });
    }
    nextMessageDecoded(expectedMessageType, template) {
        return __awaiter(this, void 0, void 0, function* () {
            return TlvObjectCodec_1.TlvObjectCodec.decode((yield this.nextMessage(expectedMessageType)).payload, template);
        });
    }
    waitForSuccess() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.nextMessage(64);
        });
    }
    send(message, type, template) {
        return __awaiter(this, void 0, void 0, function* () {
            const payload = TlvObjectCodec_1.TlvObjectCodec.encode(message, template);
            yield this.exchange.send(type, payload);
            return payload;
        });
    }
    sendError() {
        return this.sendStatusReport(1, 2);
    }
    sendSuccess() {
        return this.sendStatusReport(0, 0);
    }
    getChannelName() {
        return this.exchange.channel.channel.getName();
    }
    close() {
        this.exchange.close();
    }
    sendStatusReport(generalStatus, protocolStatus) {
        return __awaiter(this, void 0, void 0, function* () {
            const buffer = new LEBufferWriter_1.LEBufferWriter();
            buffer.writeUInt16(generalStatus);
            buffer.writeUInt32(SecureChannelMessages_1.SECURE_CHANNEL_PROTOCOL_ID);
            buffer.writeUInt16(protocolStatus);
            yield this.exchange.send(64, buffer.toBuffer());
        });
    }
    throwIfError(messageType, payload) {
        if (messageType !== 64)
            return;
        const buffer = new LEBufferReader_1.LEBufferReader(payload);
        const generalStatus = buffer.readUInt16();
        if (generalStatus === 0)
            return;
        const protocolId = buffer.readUInt32();
        const protocolStatus = buffer.readUInt16();
        throw new Error(`Received error status: ${generalStatus} ${protocolId} ${protocolStatus}`);
    }
}
exports.SecureChannelMessenger = SecureChannelMessenger;
