"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UdpMulticastServer = void 0;
const Cache_1 = require("../util/Cache");
const Network_1 = require("./Network");
class UdpMulticastServer {
    constructor(network, broadcastAddress, broadcastPort, server, restrictToInterfaceIp) {
        this.network = network;
        this.broadcastAddress = broadcastAddress;
        this.broadcastPort = broadcastPort;
        this.server = server;
        this.restrictToInterfaceIp = restrictToInterfaceIp;
        this.broadcastChannels = new Cache_1.Cache(ip => this.createBroadcastChannel(ip), 5 * 60 * 1000);
    }
    static create({ restrictToInterfaceIp, broadcastAddress, listeningPort }) {
        return __awaiter(this, void 0, void 0, function* () {
            const network = Network_1.Network.get();
            return new UdpMulticastServer(network, broadcastAddress, listeningPort, yield network.createUdpChannel({ listeningAddress: broadcastAddress, listeningPort }), restrictToInterfaceIp);
        });
    }
    onMessage(listener) {
        this.server.onData((peerAddress, port, message) => listener(message, peerAddress));
    }
    send(message, interfaceIp) {
        return __awaiter(this, void 0, void 0, function* () {
            const interfaceIps = interfaceIp !== undefined ? [interfaceIp] : this.restrictToInterfaceIp !== undefined ? [this.restrictToInterfaceIp] : this.network.getIpMacAddresses().map(({ ip }) => ip);
            yield Promise.all(interfaceIps.map((ip) => __awaiter(this, void 0, void 0, function* () { return yield (yield this.broadcastChannels.get(ip)).send(this.broadcastAddress, this.broadcastPort, message); })));
        });
    }
    createBroadcastChannel(interfaceIp) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.network.createUdpChannel({ listeningAddress: interfaceIp, listeningPort: this.broadcastPort, multicastInterface: interfaceIp });
        });
    }
    close() {
        this.server.close();
        this.broadcastChannels.close();
    }
}
exports.UdpMulticastServer = UdpMulticastServer;
