"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UdpChannelFake = void 0;
const SimulatedNetwork_1 = require("./SimulatedNetwork");
class UdpChannelFake {
    constructor(network, address, port, multicastInterface) {
        this.network = network;
        this.address = address;
        this.port = port;
        this.multicastInterface = multicastInterface;
        this.netListeners = new Array();
    }
    static create({ listeningAddress: address, listeningPort: port, multicastInterface }) {
        return __awaiter(this, void 0, void 0, function* () {
            if (address === undefined)
                throw new Error("Device IP address should be specified for fake UdpSocket");
            return new UdpChannelFake(SimulatedNetwork_1.SimulatedNetwork.get(), address, port, multicastInterface);
        });
    }
    onData(listener) {
        const netListener = this.network.onUdpData(this.address, this.port, listener);
        this.netListeners.push(netListener);
        return netListener;
    }
    send(address, port, data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.multicastInterface === undefined)
                throw new Error("Device interface should be specified to send data with a fake UdpSocket");
            this.network.sendUdp(this.multicastInterface, this.port, address, port, data);
        });
    }
    close() {
        this.netListeners.forEach(netListener => netListener.close());
        this.netListeners.length = 0;
    }
}
exports.UdpChannelFake = UdpChannelFake;
