"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NetworkNode = void 0;
const os_1 = require("os");
const UdpChannelNode_1 = require("./UdpChannelNode");
const Network_1 = require("../Network");
function ipToNumber(ip) {
    const buffer = Buffer.alloc(4);
    const ipParts = ip.split(".");
    for (var i = 0; i < 4; i++) {
        buffer.writeUInt8(parseInt(ipParts[i]), i);
    }
    return buffer.readUInt32BE();
}
class NetworkNode extends Network_1.Network {
    createUdpChannel(options) {
        return UdpChannelNode_1.UdpChannelNode.create(options);
    }
    getIpMacAddresses() {
        const result = new Array();
        const interfaces = (0, os_1.networkInterfaces)();
        for (const name in interfaces) {
            const netInterfaces = interfaces[name];
            for (const { family, mac, address } of netInterfaces) {
                if (family !== "IPv4")
                    continue;
                result.push({ ip: address, mac });
            }
        }
        return result;
    }
    getIpMacOnInterface(remoteAddress) {
        const remoteAddressNumber = ipToNumber(remoteAddress);
        const interfaces = (0, os_1.networkInterfaces)();
        for (const name in interfaces) {
            const netInterfaces = interfaces[name];
            for (const { family, mac, address, netmask } of netInterfaces) {
                if (family !== "IPv4")
                    continue;
                const netmaskNumber = ipToNumber(netmask);
                const ipNumber = ipToNumber(address);
                if ((ipNumber & netmaskNumber) !== (remoteAddressNumber & netmaskNumber))
                    continue;
                return { ip: address, mac };
            }
        }
        return undefined;
    }
}
exports.NetworkNode = NetworkNode;
